' ͻ Programmed by:
'  CLEANER.BAS  John Wilbert Villamor.
' ͼ March 14, 2011
' Copyleft 2008 - 2011 Crap Systems Inc.

'    This program can clean any text formatted BAS file by removing tab stops
' and remarks (exept for metacommands) useful for compiling without loosing
' all the precious comments and tab spaces because they occupy some base
' memory and increase the executable size.

'    This program assumes that the BAS file is created in QB where all the
' statements are uppercased since the code scanning does not automatically
' uppercase them. QuickBASIC binary files are not supported.

'    I've made this program as a supplement to compile my new game engine
' called the Sakura Engine which has a lot of blank lines, tab spaces, and
' remarks to make the code as readable as possible without wasting memory and
' as a replacement of Michael Marijuanna's crappy reduction utility. This one
' is less buggier, faster, and allows you to save the cleaned program as a
' different file so the original source is not ruined and can remove tab stops
' while Reductin can't.

'    This program should work fine in QBasic by remarking some lines near the
' beginning of the code.

' Known problems:
'  - Can't remove excess spaces in statements. Very difficult to remove
'    without ruining the code. The same goes to Reductin.
'  - Can't remove REM's at the end of statements. Newbies write programs
'    this way so its not supposed to be very important.
'  - Comments with a $ and the next letter is a capitalized S, D, or I are
'    assumed to be metacommands. Nobody should write comments that way.

DEFINT A-Z ' For the sake of speed!

' Constants to make the code more readable
CONST True = -1, False = 0

' Show the program banner
PRINT "Cleaner v1.1 by John Wilbert Villamor."
PRINT "Copyleft 2008 - 2011 Crap Systems Inc."; CHR$(13)

' Read the arguments. Remove these if you happen to use QBasic
ArgNum = 1
DIM Args$(2)
FOR ReadArgs = 1 TO LEN(COMMAND$)
	Char$ = MID$(COMMAND$, ReadArgs, 1)
	IF Char$ = CHR$(32) THEN Char$ = "": ArgNum = ArgNum + 1
	IF ArgNum = 3 THEN
		PRINT "Too many arguments!"
		END
	END IF
	Args$(ArgNum) = Args$(ArgNum) + Char$
NEXT ReadArgs
BASfile$ = Args$(1)
OUTfile$ = Args$(2)

' Check for arguments
FOR ScanArgs = 1 TO 2
	SELECT CASE Args$(ScanArgs)
		CASE "/?"
			PRINT "Syntax:"
			PRINT "CLEANER.EXE <BASFile> <OutFile>"
			PRINT "   <BASFile> - File name of the BAS file to clean."
			PRINT "   <OutFile> - Name of the cleaned BAS file."
			PRINT
			END
	END SELECT
NEXT ScanArgs

' Ask for the BAS file to clean
IF BASfile$ = "" THEN INPUT "BAS file to clean: ", BASfile$
IF BASfile$ = "" THEN END

' Open it to check it up
OPEN BASfile$ FOR BINARY AS #1
' Check for existance
IF LOF(1) = 0 THEN
	PRINT BASfile$; " not found."
	CLOSE #1: KILL BASfile$: END
END IF
' Check if its not a binary format file
IF ASC(INPUT$(1, 1)) = 252 THEN
	IF ASC(INPUT$(1, 1)) = 0 THEN
		IF ASC(INPUT$(1, 1)) = 1 THEN
			PRINT "QuickBASIC binary files are not supported!"
			CLOSE #1: END
		END IF
	END IF
END IF
CLOSE #1

' Ask for the output file
IF OUTfile$ = "" THEN INPUT "Output BAS file: ", OUTfile$
IF OUTfile$ = "" THEN END

' Make sure that the file names don't match
IF BASfile$ = OUTfile$ THEN
	PRINT "File names should not match!"
	END
END IF

' Open it for existance
OPEN OUTfile$ FOR BINARY AS #1
IF LOF(1) > 0 THEN   ' Confirm file replace if it exists
	PRINT "Do you want to replace "; OUTfile$; "? <Y/N>"
	DO
		P$ = UCASE$(INKEY$)
		IF P$ = "Y" THEN EXIT DO
		IF P$ = "N" THEN CLOSE #1: PRINT : END
	LOOP
	CLOSE #1: KILL OUTfile$
	OPEN OUTfile$ FOR BINARY AS #1
END IF
CLOSE #1

' Start the cleaning process
PRINT CHR$(13); "Cleaning...";
' Open the files
OPEN BASfile$ FOR INPUT AS #1
OPEN OUTfile$ FOR OUTPUT AS #2
' Clean the file
DO WHILE EOF(1) = 0
	LINE INPUT #1, CodeLine$   ' Grab a line

	' Remove the space on the left of the line
	CodeLine$ = LTRIM$(CodeLine$)
  
	' Remove tab spaces
	TabLength = 0
	FOR Remove = 1 TO LEN(CodeLine$)
		IF MID$(CodeLine$, Remove, 1) <> CHR$(9) THEN TabLength = Remove - 1: EXIT FOR
	NEXT Remove
	IF TabLength THEN CodeLine$ = RIGHT$(CodeLine$, LEN(CodeLine$) - TabLength)
  
	' Check for metacommands
	MetaCommand = False
	LChar$ = LEFT$(CodeLine$, 1)
	LCode$ = LEFT$(CodeLine$, 3)
	FOR Scan = 1 TO LEN(CodeLine$)
		RChar$ = MID$(CodeLine$, Scan, 1)
		IF LChar$ = "'" THEN ' Metacommand in a single quotation
			IF RChar$ = "$" THEN MetaCommand = True': EXIT FOR
		ELSEIF LCode$ = "REM" THEN ' Metacommand in a REM statement
			IF RChar$ = "$" THEN MetaCommand = True': EXIT FOR
		END IF
	NEXT Scan

	' Optimize the metacommand
	IF MetaCommand THEN
		IF LCode$ = "REM" THEN
			CodeLine$ = RIGHT$(CodeLine$, LEN(CodeLine$) - 3)
			CodeLine$ = "'" + LTRIM$(CodeLine$)
		ELSEIF LChar$ = "'" THEN
			CodeLine$ = RIGHT$(CodeLine$, LEN(CodeLine$) - 1)
			CodeLine$ = "'" + LTRIM$(CodeLine$)
		END IF
	END IF

	' Make sure that the statement is a metacommand
	SELECT CASE LEFT$(CodeLine$, 3)
		CASE "'$D": MetaCommand = True
		CASE "'$S": MetaCommand = True
		CASE "'$I": MetaCommand = True
		CASE ELSE: MetaCommand = False
	END SELECT

	' Check if the entire line is a remark or a blank
	IF MetaCommand = False THEN
		IF LEFT$(CodeLine$, 1) = "'" THEN GOTO SkipLine ' A remark
		IF CodeLine$ = "" THEN GOTO SkipLine            ' A blank
	END IF

	' Search for a remark at the end of a statement then remove it
	IF MetaCommand = False THEN
		FOR Clean = 1 TO 10
			RemovePoint = 0
			InData = False: InQuot = 0
			IF LEFT$(CodeLine$, 5) = "DATA " THEN InData = True
			FOR Scan = LEN(CodeLine$) TO 3 STEP -1
				Char$ = MID$(CodeLine$, Scan, 1)
				IF Char$ = CHR$(34) THEN InQuot = InQuot + 1 AND 1
				IF InQuot = 0 THEN
					' Remark found
					IF InData THEN
						IF Char$ = ":" THEN RemovePoint = Scan - 1: EXIT FOR
					ELSE
						IF Char$ = "'" THEN RemovePoint = Scan - 1: EXIT FOR
					END IF
				END IF
			NEXT Scan
			IF RemovePoint THEN CodeLine$ = LEFT$(CodeLine$, RemovePoint)
			CodeLine$ = RTRIM$(CodeLine$)
		NEXT Clean
	END IF
	CodeLine$ = LTRIM$(CodeLine$)
	' Skip blank lines after the cleaning just in case
	IF CodeLine$ = "" THEN GOTO SkipLine
	PRINT #2, CodeLine$

SkipLine:
LOOP
CLOSE
PRINT "Done."

OPEN BASfile$ FOR BINARY AS #1
OPEN OUTfile$ FOR BINARY AS #2

SizeLoss = LOF(1) - LOF(2)
PRINT CHR$(13); "Total code size loss:"; SizeLoss; "bytes."
END

