DEFINT A-Z
'$DYNAMIC
'$INCLUDE: 'KVESA.BI'
'$INCLUDE: 'COMMON.BI'
'$INCLUDE: 'DIRECTQB.BI'

' Sub/function declarations
DECLARE SUB ScrollLayer (Layer%, XMove%, YMove%)
DECLARE SUB DQBsbox (Layer%, X1%, Y1%, X2%, Y2%, Col%)
DECLARE SUB GETsprite (Index%, SpriteSeg%, SpriteOff%)
DECLARE FUNCTION GETmemRequired& (GETfile$)
DECLARE FUNCTION GETprocess% (GETfile$, TempLayer%, TempSeg%, TempOff%, TempPal$)

' User types
TYPE CameraType
	X AS INTEGER
	Y AS INTEGER
END TYPE

TYPE SpriteType
	X AS INTEGER
	Y AS INTEGER
	Index AS INTEGER
END TYPE

TYPE TagType
	TPos AS INTEGER
	Tags AS INTEGER
END TYPE

TYPE WindowType
	X AS INTEGER
	Y AS INTEGER
	Active AS INTEGER
	sWidth AS INTEGER
	sHeight AS INTEGER
	Title AS STRING * 20
END TYPE

' Constants
CONST ValidChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ01234567890-_"

' Defined variables
COMMON Pal AS STRING * 768
COMMON Camera AS CameraType

' Init error handler
ON ERROR GOTO EndIt

' Data arrays
TileSize = DQBsize(1, 1, 24, 24)
DIM Tile(TileSize \ 2)                       ' Tile bitmap data
DIM SHARED TileAnim(255)                     ' Tile animation flags
DIM SHARED TileFlag(255)                     ' Tile hit/draw flags
DIM TileFrame(255)                           ' Tile frame for animations
DIM AnimData(255)                            ' Animation statuses
DIM AnimRate(255)                            ' Animation rate
DIM MapData(7500)                            ' Map data
DIM Sprite(100) AS SpriteType                ' Sprite data
DIM Tags(200) AS TagType                     ' Tags data

' Arrays and variables for EMS sprite manipulation
DIM SHARED NewSpriteIndex
DIM SHARED SpriteDataOffs AS LONG
DIM SHARED SpriteSize(50)
DIM SHARED SpriteOffs(50) AS LONG
DIM SpriteBuff(DQBsize(1, 1, 128, 128) \ 2)

' Windowing system arrays
DIM SHARED Win(3) AS WindowType  ' Window coordinate

' Check files and stuff
IF FileNotFound("IMAGES\SPRITES.PCX") THEN PRINT "I cannot load IMAGES\SPRITES.PCX!": END
IF FileNotFound("ANIMEPAL.PAL") THEN PRINT "I cannot load animepal.pal!": END
IF FileNotFound("MICRO.FNT") THEN PRINT "I cannot load micro.fnt!": END

' Flat memory offsets:
' 0-148480      - 256 Tiles (580 bytes each)
' 148480-178480 - 2 layer map data (15000 bytes each)

' Init and load stuff
SpriteDataOffs = 178480
MemRequired& = GETmemRequired&("SLVEDIT.GET")
IF MemRequired& < 0 THEN
	PRINT "I cannot load slvedit.get!"
	END
END IF
IF DQBinit(4, 0, 195 + (MemRequired& \ 1024) + 1) THEN PRINT DQBerror$: END
DQBfpu
IF NOT DQBmouseDetected THEN  ' Make sure that a mouse is present
	PRINT "Microsoft compatible mouse is required."
	WHILE INKEY$ = "": WEND
	GOTO EndIt
END IF

DQBinitVGA
DQBresetMouse
'$INCLUDE: 'LOADPAL.BI'
'$INCLUDE: 'COLORS.BI'
DQBprint VIDEO, "Loading...", 1, 1, CHWHITE
Crap = DQBloadFont("MICRO.FNT")
DQBclearLayer VIDEO
SetFile$ = "TILESETS\CRAPLAND.STS": GOSUB LoadSet
IF Fail THEN GOTO EndIt
Backdrop$ = "IMAGES\NIGHTSKY.PCX": GOSUB LoadBackdrop

' Load and clear data
Msgbox VIDEO, "Loading sprites..."
Crap = GETprocess("SLVEDIT.GET", 4, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0)), Pal)
IF Crap THEN Msgbox VIDEO, "I cannot load sprite data!": WHILE INKEY$ = "": WEND: GOTO EndIt
Msgbox VIDEO, "Clearing level data..."
GOSUB Clearlevel

' Default editor settings
ScrollMode = 0
ScrollRate = 0
Camera.X = 0
Camera.Y = 0
MapXDim = 13
MapYDim = 8
SpriteCount = 0
TagCount = 0
ViewLayer = 0
CurLayer = 0
Grid = 1

' Stat window coordinates
Win(1).Active = 1
Win(1).Title = "Editor/Level Stats"
Win(1).sWidth = 121
Win(1).sHeight = 58
Win(1).X = 319 - Win(1).sWidth
Win(1).Y = 0

' File names window coordinates
Win(2).Active = 1
Win(2).Title = "File Names"
Win(2).sWidth = 118
Win(2).sHeight = 28
Win(2).X = 319 - Win(2).sWidth
Win(2).Y = Win(1).sHeight + 1

' Tile window coordinates
Win(3).Active = 1
Win(3).Title = "Current Tile"
Win(3).sWidth = 68
Win(3).sHeight = 43
Win(3).X = 319 - Win(3).sWidth
Win(3).Y = Win(2).Y + (Win(2).sHeight + 1)

MainLoop:
DO
	Press$ = UCASE$(INKEY$)

	' Scroll around
	IF DQBmouseRB THEN
		FOR Scroll = 1 TO 10
			' Scroll right
			IF DQBmouseX > 160 + 10 * Scroll AND Camera.X < (24 * MapXDim) - 320 THEN
				IF ScrollMode = 1 OR ScrollMode = 3 THEN HScrollCount = HScrollCount + 1
				IF HScrollCount > ScrollRate THEN
					IF ScrollMode = 1 OR ScrollMode = 3 THEN ScrollLayer 2, -1, 0
					HScrollCount = 0
				END IF
				DQBsetMousePos DQBmouseX - 1, DQBmouseY
				Camera.X = Camera.X + 1
			END IF
			' Scroll left
			IF DQBmouseX < 160 - 10 * Scroll AND Camera.X > 0 THEN
				IF ScrollMode = 1 OR ScrollMode = 3 THEN HScrollCount = HScrollCount + 1
				IF HScrollCount > ScrollRate THEN
					IF ScrollMode = 1 OR ScrollMode = 3 THEN ScrollLayer 2, 1, 0
					HScrollCount = 0
				END IF
				DQBsetMousePos DQBmouseX + 1, DQBmouseY
				Camera.X = Camera.X - 1
			END IF
			' Scroll down
			IF DQBmouseY > 100 + 10 * Scroll AND Camera.Y < (24 * MapYDim) - 200 THEN
				IF ScrollMode = 2 OR ScrollMode = 3 THEN VScrollCount = VScrollCount + 1
				IF VScrollCount > ScrollRate THEN
					IF ScrollMode = 2 OR ScrollMode = 3 THEN ScrollLayer 2, 0, -1
					VScrollCount = 0
				END IF
				DQBsetMousePos DQBmouseX, DQBmouseY - 1
				Camera.Y = Camera.Y + 1
			END IF
			' Scroll up
			IF DQBmouseY < 100 - 10 * Scroll AND Camera.Y > 0 THEN
				IF ScrollMode = 2 OR ScrollMode = 3 THEN VScrollCount = VScrollCount + 1
				IF VScrollCount > ScrollRate THEN
					IF ScrollMode = 2 OR ScrollMode = 3 THEN ScrollLayer 2, 0, 1
					VScrollCount = 0
				END IF
				DQBsetMousePos DQBmouseX, DQBmouseY + 1
				Camera.Y = Camera.Y - 1
			END IF
		NEXT Scroll
	END IF
	CamX = Camera.X - Camera.X * 2
	CamY = Camera.Y - Camera.Y * 2

	' Calculate start and end tile plotting points
	StartX = Camera.X \ 24
	StartY = Camera.Y \ 24
	IF StartX < 1 THEN StartX = 1
	IF StartY < 1 THEN StartY = 1
	EndX = StartX + 15
	EndY = StartY + 10
	IF EndX > MapXDim THEN EndX = MapXDim
	IF EndY > MapYDim THEN EndY = MapYDim
  
	' Draw background tiles
	DQBcopyLayer 2, 1
	FOR Layer = 1 TO 0 STEP -1
		DrawIt = FALSE
		SELECT CASE ViewLayer
			CASE 0: DrawIt = TRUE
			CASE 1: IF Layer = 0 THEN DrawIt = TRUE
			CASE 2: IF Layer = 1 THEN DrawIt = TRUE
		END SELECT
		IF DrawIt THEN
			DQBpeek VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * Layer), 15000
			FOR dy = StartY TO EndY
				FOR dx = StartX TO EndX
					BlockX = 24 * (dx - 1) - Camera.X
					BlockY = 24 * (dy - 1) - Camera.Y
					Offs = (dx - 1) + 150 * (dy - 1)
					' Get and draw tile
					DEF SEG = VARSEG(MapData(0))
					Tile = PEEK(VARPTR(MapData(0)) + Offs)
					DEF SEG = VARSEG(TileFlag(Tile))
					DrawFlag = PEEK(VARPTR(TileFlag(Tile)) + 1)
					DEF SEG
					SELECT CASE DrawFlag
						CASE 1: DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * (Tile + TileFrame(Tile)), TileSize: RELsprite 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
						CASE 3: DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * (Tile + TileFrame(Tile)), TileSize: RELspriteTrans 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
					END SELECT
					' Draw grid
					IF Grid AND Layer = 0 THEN
						DQBline 1, BlockX, BlockY, BlockX + 23, BlockY, CWHITE
						DQBline 1, BlockX, BlockY, BlockX, BlockY + 23, CWHITE
						DQBline 1, BlockX + 23, BlockY, BlockX + 23, BlockY + 23, CWHITE
						DQBline 1, BlockX, BlockY + 23, BlockX + 23, BlockY + 23, CWHITE
					END IF
				NEXT dx
			NEXT dy
		END IF
	NEXT Layer
  
	' Drag sprite if selected
	IF DQBmouseLB AND OnSprite THEN
		Sprite(SelSprite).X = DQBmouseX + Camera.X
		Sprite(SelSprite).Y = DQBmouseY + Camera.Y
		Changed = TRUE
	END IF
	' Put sprites
	SelSprite = 0
	OnSprite = FALSE
	FOR S = 1 TO SpriteCount
		IF Sprite(S).Index THEN
			GETsprite Sprite(S).Index, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			RELsprite 1, Sprite(S).X + CamX, Sprite(S).Y + CamY, VARSEG(SpriteBuff(0)), VARPTR(SpriteBuff(0))
			' Check if a sprite is selected
			DQBprint 1, LTRIM$(RTRIM$(STR$(Sprite(S).Index))), (Sprite(S).X + CamX) - 1, (Sprite(S).Y + CamY) - 2, CHWHITE
			IF DQBmouseX >= ((Sprite(S).X - 5) + CamX) AND DQBmouseX <= ((Sprite(S).X + 5) + CamX) THEN
				IF DQBmouseY >= ((Sprite(S).Y - 5) + CamY) AND DQBmouseY <= ((Sprite(S).Y + 5) + CamY) THEN
					DQBsbox 1, (Sprite(S).X - 5) + CamX, (Sprite(S).Y - 5) + CamY, (Sprite(S).X + 5) + CamX, (Sprite(S).Y + 5) + CamY, CRED
					OnSprite = TRUE
					SelSprite = S
				END IF
			END IF
		END IF
	NEXT S
  
	' Draw foreground tiles
	FOR Layer = 1 TO 0 STEP -1
		DrawIt = FALSE
		SELECT CASE ViewLayer
			CASE 0: DrawIt = TRUE
			CASE 1: IF Layer = 0 THEN DrawIt = TRUE
			CASE 2: IF Layer = 1 THEN DrawIt = TRUE
		END SELECT
		IF DrawIt THEN
			DQBpeek VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * Layer), 15000
			FOR dy = StartY TO EndY
				FOR dx = StartX TO EndX
					BlockX = 24 * (dx - 1) - Camera.X
					BlockY = 24 * (dy - 1) - Camera.Y
					Offs = (dx - 1) + 150 * (dy - 1)
					' Get and draw tile
					DEF SEG = VARSEG(MapData(0))
					Tile = PEEK(VARPTR(MapData(0)) + Offs)
					DEF SEG = VARSEG(TileFlag(Tile))
					DrawFlag = PEEK(VARPTR(TileFlag(Tile)) + 1)
					DEF SEG
					SELECT CASE DrawFlag
						CASE 0: DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * (Tile + TileFrame(Tile)), TileSize: RELsprite 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
						CASE 2: DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * (Tile + TileFrame(Tile)), TileSize: RELspriteTrans 1, BlockX, BlockY, VARSEG(Tile(0)), VARPTR(Tile(0))
					END SELECT
					' Do matrix search for current mouse position
					IF Layer = CurLayer THEN
						IF DQBmouseX >= BlockX + 1 AND DQBmouseX <= BlockX + 24 THEN
							IF DQBmouseY >= BlockY + 1 AND DQBmouseY <= BlockY + 24 THEN
								CurX = dx
								CurY = dy
								CurOffs = Offs
								IF CurLayer = 0 THEN DQBsbox 1, BlockX + 1, BlockY + 1, BlockX + 22, BlockY + 22, CYELLOW
								IF CurLayer = 1 THEN DQBsbox 1, BlockX + 1, BlockY + 1, BlockX + 22, BlockY + 22, CHGREEN
							END IF
						END IF
					END IF
				NEXT dx
			NEXT dy
		END IF
	NEXT Layer

	' Put tags
	OnTag = FALSE
	FOR P = 1 TO TagCount
		DEF SEG = VARSEG(Tags(P))
		TagX = PEEK(VARPTR(Tags(P)) + 0)
		TagY = PEEK(VARPTR(Tags(P)) + 1)
		HTag = PEEK(VARPTR(Tags(P)) + 2)
		LTag = PEEK(VARPTR(Tags(P)) + 3)
		DEF SEG
		TileX = 24 * TagX
		TileY = 24 * TagY
		DQBprint 1, "H:" + LTRIM$(STR$(HTag)), (TileX + 2) + CamX, (TileY + 2) + CamY, CHWHITE
		DQBprint 1, "L:" + LTRIM$(STR$(LTag)), (TileX + 2) + CamX, (TileY + 8) + CamY, CHWHITE
		IF TagX + 1 = CurX AND TagY + 1 = CurY THEN OnTag = TRUE: SelTag = P
	NEXT P

	' Editor keys
	IF Press$ = CHR$(0) + "?" THEN ViewLayer = 1
	IF Press$ = CHR$(0) + "@" THEN ViewLayer = 2
	IF Press$ = CHR$(0) + "A" THEN ViewLayer = 0
	IF Press$ = CHR$(0) + ";" THEN Win(1).Active = Win(1).Active + 1 AND 1
	IF Press$ = CHR$(0) + "<" THEN Win(2).Active = Win(2).Active + 1 AND 1
	IF Press$ = CHR$(0) + "=" THEN Win(3).Active = Win(3).Active + 1 AND 1
	IF Press$ = "G" THEN Grid = Grid + 1 AND 1
	IF Press$ = "V" THEN GOSUB PickTile
	IF Press$ = "+" AND CurTile < 255 THEN CurTile = CurTile + 1
	IF Press$ = "-" AND CurTile > 0 THEN CurTile = CurTile - 1
	IF Press$ = CHR$(13) THEN CurLayer = CurLayer + 1 AND 1
	' Editing keys
	IF Press$ = "8" AND MapYDim > 1 THEN MapYDim = MapYDim - 1: Changed = TRUE
	IF Press$ = "2" AND MapYDim < 100 THEN MapYDim = MapYDim + 1: Changed = TRUE
	IF Press$ = "4" AND MapXDim > 1 THEN MapXDim = MapXDim - 1: Changed = TRUE
	IF Press$ = "6" AND MapXDim < 150 THEN MapXDim = MapXDim + 1: Changed = TRUE
	IF Press$ = CHR$(0) + "R" AND TagCount < 200 THEN  ' Insert tile tag
		' Check if the cursor is on another tag
		DontPut = FALSE
		FOR c = 1 TO TagCount
			DEF SEG = VARSEG(Tags(c))
			TagX = PEEK(VARPTR(Tags(c)) + 0) + 1
			TagY = PEEK(VARPTR(Tags(c)) + 1) + 1
			DEF SEG
			IF TagX = CurX AND TagY = CurY THEN DontPut = TRUE: EXIT FOR
		NEXT c
		IF DontPut = FALSE THEN
			DEF SEG = VARSEG(Tags(TagCount + 1))
			POKE VARPTR(Tags(TagCount + 1)) + 0, CurX - 1
			POKE VARPTR(Tags(TagCount + 1)) + 1, CurY - 1
			POKE VARPTR(Tags(TagCount + 1)) + 2, 0
			POKE VARPTR(Tags(TagCount + 1)) + 3, 0
			DEF SEG
			TagCount = TagCount + 1
			Changed = TRUE
		END IF
	END IF
	IF Press$ = "S" AND SpriteCount < 100 THEN    ' Insert sprite
		Sprite(SpriteCount + 1).X = DQBmouseX + Camera.X
		Sprite(SpriteCount + 1).Y = DQBmouseY + Camera.Y
		Sprite(SpriteCount + 1).Index = 1
		SpriteCount = SpriteCount + 1
		Changed = TRUE
	END IF
	IF Press$ = CHR$(0) + "S" THEN   ' Delete
		IF OnSprite THEN  ' Delete sprite
			FOR DelSprite = 1 TO SpriteCount
				IF DelSprite > SelSprite THEN
					Sprite(DelSprite - 1).X = Sprite(DelSprite).X
					Sprite(DelSprite - 1).Y = Sprite(DelSprite).Y
					Sprite(DelSprite - 1).Index = Sprite(DelSprite).Index
				END IF
			NEXT DelSprite
			SpriteCount = SpriteCount - 1
			Changed = TRUE
		ELSEIF OnTag THEN ' Delete tag
			FOR DelTag = 1 TO TagCount
				IF DelTag > SelTag THEN
					Tags(DelTag - 1) = Tags(DelTag)
				END IF
			NEXT DelTag
			TagCount = TagCount - 1
			Changed = TRUE
		END IF
	END IF
	IF Press$ = "/" AND ScrollRate = 0 THEN
		ScrollMode = ScrollMode + 1
		IF ScrollMode > 3 THEN ScrollMode = 0
		Changed = TRUE
	ELSEIF Press$ = "/" THEN
		ScrollRate = ScrollRate - 1
		Changed = TRUE
	END IF
	IF Press$ = "*" AND ScrollMode THEN ScrollRate = ScrollRate + 1: Changed = TRUE

	' Window handling code
	DQBprint 1, "FPS:" + LTRIM$(STR$(FPS)), 1, 1, CHWHITE
	FOR Drag = 3 TO 1 STEP -1
		IF DQBmouseLB AND Plotting = FALSE AND OnSprite = FALSE THEN
			IF Win(Drag).Active THEN
				IF DQBmouseX >= Win(Drag).X AND DQBmouseX <= Win(Drag).X + Win(Drag).sWidth THEN
					IF DQBmouseY >= Win(Drag).Y AND DQBmouseY <= Win(Drag).Y + Win(Drag).sHeight THEN
						DQBcopyLayer 1, 4
						ClickedX = DQBmouseX - Win(Drag).X
						ClickedY = DQBmouseY - Win(Drag).Y
						' Drag window
						DQBsetMouseRange ClickedX, ClickedY, (319 - Win(Drag).sWidth) + ClickedX, (199 - Win(Drag).sHeight) + ClickedY
						DO WHILE DQBmouseLB
							DQBcopyLayer 4, 1
							Win(Drag).X = DQBmouseX - ClickedX
							Win(Drag).Y = DQBmouseY - ClickedY
							GOSUB DrawWindows
							GOSUB DrawCursor
							DQBcopyLayer 1, VIDEO
							DQBwait 1
						LOOP
						DQBsetMouseRange 0, 0, 319, 199
						' Close window
						IF ClickedX > Win(Drag).sWidth - 13 AND ClickedX < Win(Drag).sWidth - 1 THEN
							IF ClickedY > 2 AND ClickedY < 8 THEN
								Win(Drag).Active = 0
							END IF
						END IF
					END IF
				END IF
			END IF
		END IF
	NEXT Drag
	GOSUB DrawWindows

	' Plot tiles
	Plotting = FALSE
	IF DQBmouseLB AND OnSprite = FALSE THEN
		DQBpeek VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * CurLayer), 15000
		DEF SEG = VARSEG(MapData(0))
		POKE VARPTR(MapData(0)) + CurOffs, CurTile
		DEF SEG
		Plotting = TRUE
		Changed = TRUE
		DQBpoke VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * CurLayer), 15000
	END IF
	' Get tile under cursor
	IF Press$ = CHR$(9) THEN
		DQBpeek VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * CurLayer), 15000
		DEF SEG = VARSEG(MapData(0))
		CurTile = PEEK(VARPTR(MapData(0)) + CurOffs)
		DEF SEG
	END IF

	' Tag value manipulation
	IF OnTag THEN
		IF Press$ = "H" THEN ' Set hi-tag value
			DEF SEG = VARSEG(Tags(SelTag))
			HTag = PEEK(VARPTR(Tags(SelTag)) + 2)
			DEF SEG
			NewTag$ = InputBox$("Enter hi-tag value (0-255)", "0123456789", LTRIM$(RTRIM$(STR$(HTag))), 3, 1, 4, 0)
			IF VAL(NewTag$) > 255 THEN
				Msgbox VIDEO, "Invalid tag value!"
				WHILE INKEY$ = "": WEND
			ELSEIF NewTag$ <> "" THEN
				HTag = VAL(NewTag$)
				DEF SEG = VARSEG(Tags(SelTag))
				POKE VARPTR(Tags(SelTag)) + 2, HTag
				DEF SEG
				Changed = TRUE
			END IF
		END IF
		IF Press$ = "T" THEN ' Set lo-tag value
			DEF SEG = VARSEG(Tags(SelTag))
			LTag = PEEK(VARPTR(Tags(SelTag)) + 3)
			DEF SEG
			NewTag$ = InputBox$("Enter lo-tag value (0-255)", "0123456789", LTRIM$(RTRIM$(STR$(LTag))), 3, 1, 4, 0)
			IF VAL(NewTag$) > 255 THEN
				Msgbox VIDEO, "Invalid tag value!"
				WHILE INKEY$ = "": WEND
			ELSEIF NewTag$ <> "" THEN
				LTag = VAL(NewTag$)
				DEF SEG = VARSEG(Tags(SelTag))
				POKE VARPTR(Tags(SelTag)) + 3, LTag
				DEF SEG
				Changed = TRUE
			END IF
		END IF
	END IF
	' Sprite index manipulation
	IF OnSprite THEN
		IF Press$ = "I" THEN
			NewIndex$ = InputBox$("Enter sprite index (1-255)", "0123456789", LTRIM$(RTRIM$(STR$(Sprite(SelSprite).Index))), 3, 1, 4, 0)
			IF VAL(NewIndex$) < 1 OR VAL(NewIndex$) > 255 THEN
				Msgbox VIDEO, "Invalid index value!"
				WHILE INKEY$ = "": WEND
			ELSEIF NewIndex$ <> "" THEN
				Sprite(SelSprite).Index = VAL(NewIndex$)
				Changed = TRUE
			END IF
		END IF
	END IF
  
	' Save and load related keys
	IF Press$ = CHR$(2) THEN GOSUB ChangeBackdrop
	IF Press$ = CHR$(20) THEN GOSUB ChangeTileset
	IF Press$ = CHR$(14) THEN GOSUB NewLevel
	IF Press$ = CHR$(19) THEN GOSUB SaveLevel
	IF Press$ = CHR$(12) THEN GOSUB LoadLevel
	IF Press$ = CHR$(0) + ">" THEN GOSUB ViewLevel

	' Update screen
	GOSUB DrawCursor
	DQBcopyLayer 1, VIDEO
	DQBwait 1

	' Do animations
	FOR Animate = 0 TO 255
		' Get animation flags
		DEF SEG = VARSEG(TileAnim(Animate))
		Anim = PEEK(VARPTR(TileAnim(Animate)))
		AnmF = PEEK(VARPTR(TileAnim(Animate)) + 1)
		' Get animation stats
		DEF SEG = VARSEG(AnimData(Animate))
		AnimDir = PEEK(VARPTR(AnimData(Animate))) - 1
		AnimCnt = PEEK(VARPTR(AnimData(Animate)) + 1)
		AnimCnt = AnimCnt + 1
		IF AnimCnt > AnimRate(Animate) THEN
			SELECT CASE Anim
				CASE 1
					IF TileFrame(Animate) = 0 THEN AnimDir = 1
					IF TileFrame(Animate) = AnmF THEN AnimDir = -1
					TileFrame(Animate) = TileFrame(Animate) + AnimDir
				CASE 2
					TileFrame(Animate) = TileFrame(Animate) + 1
					IF TileFrame(Animate) > AnmF THEN TileFrame(Animate) = 0
				CASE 3
					TileFrame(Animate) = TileFrame(Animate) - 1
					IF TileFrame(Animate) < (AnmF - AnmF * 2) THEN TileFrame(Animate) = 0
			END SELECT
			AnimCnt = 1
		END IF
		' Set animation stats
		POKE VARPTR(AnimData(Animate)), AnimDir + 1
		POKE VARPTR(AnimData(Animate)) + 1, AnimCnt
		DEF SEG
	NEXT Animate

	' Calculate frame rate
	Loops = Loops + 1
	IF CLNG(TIMER) > OldTime& THEN
		OldTime& = CLNG(TIMER)
		FPS = Loops
		Loops = 0
	END IF

LOOP UNTIL Press$ = CHR$(27)
Msgbox VIDEO, "Are you sure you want to quit? <Y/N>"
DO
	P$ = UCASE$(INKEY$)
	IF P$ = "Y" THEN EXIT DO
	IF P$ = "N" GOTO MainLoop
LOOP
IF Changed THEN
	Msgbox VIDEO, "Save changes? <Y/N>"
	DO
		P$ = UCASE$(INKEY$)
		IF P$ = "Y" THEN GOSUB SaveLevel: EXIT DO
		IF P$ = "N" GOTO EndIt
	LOOP
END IF
EndIt:
DQBclose
IF ERR THEN PRINT "Some stupid error #"; LTRIM$(STR$(ERR))
END
RESUME

DrawWindows:
	IF Win(1).Active THEN GOSUB DrawStats
	IF Win(2).Active THEN GOSUB DrawFiles
	IF Win(3).Active THEN GOSUB DrawCurTile
RETURN

DrawStats:
	Index = 1: GOSUB DrawWindow
	DQBprint 1, "Cursor X:" + LTRIM$(RTRIM$(STR$(CurX))) + "/" + LTRIM$(STR$(MapXDim)), Win(1).X + 2, Win(1).Y + 10, CHWHITE
	DQBprint 1, "Cursor Y:" + LTRIM$(RTRIM$(STR$(CurY))) + "/" + LTRIM$(STR$(MapYDim)), Win(1).X + 2, Win(1).Y + 16, CHWHITE
	DQBprint 1, "Offset:" + LTRIM$(STR$(CurOffs)), Win(1).X + 2, Win(1).Y + 22, CHWHITE
	DQBprint 1, "Camera X:" + LTRIM$(RTRIM$(STR$(Camera.X))), Win(1).X + 69, Win(1).Y + 10, CHWHITE
	DQBprint 1, "Camera Y:" + LTRIM$(RTRIM$(STR$(Camera.Y))), Win(1).X + 69, Win(1).Y + 16, CHWHITE
	SELECT CASE ScrollMode
		CASE 0: ScrollMode$ = "Off"
		CASE 1: ScrollMode$ = "Horizontal"
		CASE 2: ScrollMode$ = "Vertical"
		CASE 3: ScrollMode$ = "Both"
	END SELECT
	DQBprint 1, "Scroll Mode:" + ScrollMode$, Win(1).X + 2, Win(1).Y + 34, CHWHITE
	DQBprint 1, "Scroll Rate:" + LTRIM$(RTRIM$(STR$(ScrollRate))), Win(1).X + 2, Win(1).Y + 40, CHWHITE
	DQBprint 1, "Sprites:" + LTRIM$(RTRIM$(STR$(SpriteCount))) + "/100", Win(1).X + 2, Win(1).Y + 52, CHWHITE
	DQBprint 1, "Tags:" + LTRIM$(RTRIM$(STR$(TagCount))) + "/200", Win(1).X + 69, Win(1).Y + 52, CHWHITE
RETURN

DrawFiles:
	Index = 2: GOSUB DrawWindow
	DQBprint 1, "Backdrop:" + Backdrop$, Win(Index).X + 2, Win(Index).Y + 10, CHWHITE
	DQBprint 1, "Tileset:" + SetFile$, Win(Index).X + 2, Win(Index).Y + 16, CHWHITE
	DQBprint 1, "Level:" + LevelFile$, Win(Index).X + 2, Win(Index).Y + 22, CHWHITE
RETURN

DrawCurTile:
	Index = 3: GOSUB DrawWindow
	DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * (CurTile + TileFrame(CurTile)), TileSize
	DQBboxf 1, Win(3).X + 2, Win(3).Y + 10, Win(3).X + 27, Win(3).Y + 35, CGREY
	DQBprint 1, "Tile:" + LTRIM$(RTRIM$(STR$(CurTile))) + "/255", Win(3).X + 2, Win(3).Y + 37, CHWHITE
	DQBprint 1, "HF:" + LTRIM$(STR$(HitF)), Win(3).X + 29, Win(3).Y + 10, CHWHITE
	DQBprint 1, "DF:" + LTRIM$(STR$(DrwF)), Win(3).X + 29, Win(3).Y + 16, CHWHITE
	DEF SEG = VARSEG(TileFlag(CurTile))
	HitF = PEEK(VARPTR(TileAnim(CurTile)))
	DrwF = PEEK(VARPTR(TileAnim(CurTile)) + 1)
	DEF SEG
	SELECT CASE DrwF
		CASE 0 TO 1: RELsprite 1, Win(3).X + 3, Win(3).Y + 11, VARSEG(Tile(0)), VARPTR(Tile(0))
		CASE 2 TO 3: RELspriteTrans 1, Win(3).X + 3, Win(3).Y + 11, VARSEG(Tile(0)), VARPTR(Tile(0))
	END SELECT
RETURN

DrawWindow:
	DQBbox 1, Win(Index).X, Win(Index).Y, Win(Index).X + Win(Index).sWidth, Win(Index).Y + Win(Index).sHeight, CGREY
	DQBline 1, Win(Index).X, Win(Index).Y, Win(Index).X + Win(Index).sWidth, Win(Index).Y, CHWHITE
	DQBline 1, Win(Index).X, Win(Index).Y, Win(Index).X, Win(Index).Y + Win(Index).sHeight, CHWHITE
	DQBboxf 1, Win(Index).X + 1, Win(Index).Y + 1, Win(Index).X + (Win(Index).sWidth - 1), Win(Index).Y + (Win(Index).sHeight - 1), CWHITE
	DQBboxf 1, Win(Index).X + 2, Win(Index).Y + 2, Win(Index).X + (Win(Index).sWidth - 2), Win(Index).Y + 8, CBLUE
	DQBprint 1, Win(Index).Title, Win(Index).X + 3, Win(Index).Y + 3, CHWHITE
	DQBprint 1, "[X]", (Win(Index).X + Win(Index).sWidth) - 13, Win(Index).Y + 3, CHWHITE
RETURN

DrawCursor:
	DQBline 1, DQBmouseX - 5, DQBmouseY, DQBmouseX - 1, DQBmouseY, CYELLOW
	DQBline 1, DQBmouseX + 5, DQBmouseY, DQBmouseX + 1, DQBmouseY, CYELLOW
	DQBline 1, DQBmouseX, DQBmouseY - 5, DQBmouseX, DQBmouseY - 1, CYELLOW
	DQBline 1, DQBmouseX, DQBmouseY + 5, DQBmouseX, DQBmouseY + 1, CYELLOW
RETURN

NewLevel:
	Msgbox VIDEO, "New level? <Y/N>"
	DO
		P$ = UCASE$(INKEY$)
		IF P$ = "Y" THEN EXIT DO
		IF P$ = "N" THEN RETURN
	LOOP
	GOSUB Clearlevel
	LevelFile$ = ""
	ScrollMode = 0
	ScrollRate = 0
	Camera.X = 0
	Camera.Y = 0
	MapXDim = 13
	MapYDim = 8
	SpriteCount = 0
	TagCount = 0
	ViewLayer = 0
	CurLayer = 0
RETURN

LoadLevel:
	Level2Load$ = FileSelect$("LEVELS\*.SLV", "Sakura Level File", "Load level", 1, 4)
	IF Level2Load$ = "" THEN RETURN
	Level2Load$ = "LEVELS\" + Level2Load$
	' Start loading
	Msgbox VIDEO, "Loading " + Level2Load$ + "..."
	OPEN Level2Load$ FOR BINARY AS #1
	' Check for the ID marker
	IF INPUT$(4, 1) <> "SLV1" THEN
		Msgbox VIDEO, Level2Load$ + " is not a valid Sakura Level file!"
		WHILE INKEY$ = "": WEND
		RETURN
	END IF
	' Clear current level
	GOSUB Clearlevel
	' Get map values
	MapXDim = ASC(INPUT$(1, 1))
	MapYDim = ASC(INPUT$(1, 1))
	ScrollMode = ASC(INPUT$(1, 1))
	ScrollRate = ASC(INPUT$(1, 1))
	SpriteCount = ASC(INPUT$(1, 1))
	TagCount = ASC(INPUT$(1, 1))
	CBackdrop$ = INPUT$(ASC(INPUT$(1, 1)), 1)
	CSetFile$ = INPUT$(ASC(INPUT$(1, 1)), 1)
	' Load level
	FOR ReadLayer = 0 TO 1
		DEF SEG = VARSEG(MapData(0))
		FOR ry = 1 TO MapYDim
			Buffer$ = INPUT$(MapXDim, 1)
			FOR rx = 1 TO MapXDim
				Num = ASC(MID$(Buffer$, rx, 1))
				Offs = (rx - 1) + 150 * (ry - 1)
				POKE VARPTR(MapData(0)) + Offs, Num
			NEXT rx
		NEXT ry
		DEF SEG
		DQBpoke VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * ReadLayer), 15000
	NEXT ReadLayer
	' Read sprite data
	FOR r = 1 TO SpriteCount: GET #1, , Sprite(r): NEXT
	' Read tag data
	FOR r = 1 TO TagCount: GET #1, , Tags(r): NEXT
	CLOSE #1
	LevelFile$ = Level2Load$
	IF CBackdrop$ <> Backdrop$ THEN Backdrop$ = CBackdrop$: GOSUB LoadBackdrop
	IF CSetFile$ <> SetFile$ THEN SetFile$ = CSetFile$: GOSUB LoadSet
	Changed = FALSE
	Camera.X = 0
	Camera.Y = 0
RETURN

SaveLevel:
	IF LevelFile$ = "" THEN ' Ask for new level file name
		NewLevelFile$ = InputBox$("New level file name", ValidChars, "UNTITLED", 8, 1, 4, 1)
		IF NewLevelFile$ = "" THEN RETURN
		NewLevelFile$ = "LEVELS\" + NewLevelFile$ + ".SLV"
		IF NOT FileNotFound(NewLevelFile$) THEN   ' Confirm overwrite
			Msgbox VIDEO, "File already exists! Overwrite? <Y/N>"
			DO
				P$ = UCASE$(INKEY$)
				IF P$ = "Y" THEN EXIT DO
				IF P$ = "N" THEN RETURN
			LOOP
		END IF
		LevelFile$ = NewLevelFile$
	END IF
	' Start saving
	Msgbox VIDEO, "Saving as " + LevelFile$ + "..."
	IF NOT FileNotFound(LevelFile$) THEN KILL LevelFile$
	OPEN LevelFile$ FOR BINARY AS #1
	' Generate map header and write it
	Hdr$ = "SLV1" + CHR$(MapXDim) + CHR$(MapYDim)
	Hdr$ = Hdr$ + CHR$(ScrollMode) + CHR$(ScrollRate)
	Hdr$ = Hdr$ + CHR$(SpriteCount) + CHR$(TagCount)
	Hdr$ = Hdr$ + CHR$(LEN(Backdrop$)) + Backdrop$
	Hdr$ = Hdr$ + CHR$(LEN(SetFile$)) + SetFile$
	PUT #1, , Hdr$
	' Write map data
	FOR WriteLayer = 0 TO 1
		DQBpeek VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * WriteLayer), 15000
		DEF SEG = VARSEG(MapData(0))
		FOR Wy = 1 TO MapYDim
			Buffer$ = ""
			FOR Wx = 1 TO MapXDim
				Offs = (Wx - 1) + 150 * (Wy - 1)
				Buffer$ = Buffer$ + CHR$(PEEK(VARPTR(MapData(0)) + Offs))
			NEXT Wx
			PUT #1, , Buffer$
		NEXT Wy
		DEF SEG
	NEXT WriteLayer
	' Write sprite data
	FOR W = 1 TO SpriteCount: PUT #1, , Sprite(W): NEXT
	' Write tag data
	FOR W = 1 TO TagCount: PUT #1, , Tags(W): NEXT
	CLOSE #1
	Changed = FALSE
RETURN

ChangeTileset:
	NewSetFile$ = FileSelect$("TILESETS\*.STS", "Sakura Tileset", "Select Tileset", 1, 4)
	IF NewSetFile$ = "" THEN RETURN
	OldSet$ = SetFile$: SetFile$ = "TILESETS\" + NewSetFile$
	GOSUB LoadSet
	IF Fail THEN SetFile$ = OldSet$
	IF Fail = FALSE THEN Changed = TRUE
RETURN

ChangeBackdrop:
	NewBackdrop$ = FileSelect$("IMAGES\*.BMP", "Bitmap Image", "Backdrop image", 1, 4)
	IF NewBackdrop$ = "" THEN NewBackdrop$ = FileSelect$("IMAGES\*.PCX", "PCX Image", "Backdrop image", 1, 4)
	IF NewBackdrop$ = "" THEN RETURN
	NewBackdrop$ = "IMAGES\" + NewBackdrop$
	Crap = DQBloadImage(2, 0, 0, NewBackdrop$, Pal, 0, 0)
	IF Crap THEN
		Msgbox VIDEO, "I cannot load " + NewBackdrop$ + "!"
		WHILE INKEY$ = "": WEND
		Crap = DQBloadImage(2, 0, 0, Backdrop$, Pal, 0, 0)
		RETURN
	END IF
	Backdrop$ = NewBackdrop$
	Changed = TRUE
RETURN

PickTile:
	FOR SelY = 1 TO 25
		FOR SelX = 1 TO 12
			Index = (SelX - 1) + 12 * (SelY - 1)
			IF Index = CurTile THEN EXIT FOR
		NEXT SelX
		IF Index = CurTile THEN EXIT FOR
	NEXT SelY
	DO
		P$ = UCASE$(INKEY$)
		DQBclearLayer 1
		Tx = 1
		Ty = (ListY - ListY * 2) + 1
		Index = 0
		FOR dy = 1 TO 25
			FOR dx = 1 TO 12
				DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * Index, TileSize
				RELsprite 1, Tx, Ty, VARSEG(Tile(0)), VARPTR(Tile(0))
				DQBprint 1, LTRIM$(STR$(Index)), Tx + 2, Ty + 2, CHWHITE
				Tx = Tx + 25
				Index = Index + 1
				IF Index > 255 THEN EXIT FOR
			NEXT dx
			Tx = 1
			Ty = Ty + 25
			IF Index > 255 THEN EXIT FOR
		NEXT dy
		IF P$ = CHR$(0) + "H" AND SelY > 1 THEN SelY = SelY - 1
		IF P$ = CHR$(0) + "P" AND SelY < 22 THEN SelY = SelY + 1
		IF P$ = CHR$(0) + "K" AND SelX > 1 THEN SelX = SelX - 1
		IF P$ = CHR$(0) + "M" AND SelX < 12 THEN SelX = SelX + 1
		IF (SelX - 1) + 12 * (SelY - 1) > 255 THEN SelX = 4: SelY = 22
		Index = (SelX - 1) + 12 * (SelY - 1)
		DQBsbox 1, 25 * (SelX - 1), (25 * (SelY - 1)) - ListY, (25 * (SelX - 1)) + 25, ((25 * (SelY - 1)) + 25) - ListY, CYELLOW
		DQBcopyLayer 1, VIDEO
		DQBwait 1
		FOR Scroll = 1 TO 20
			IF (25 * (SelY - 1)) + 12 < (ListY + 100) - 10 * Scroll THEN ListY = ListY - 1
			IF (25 * (SelY - 1)) + 12 > (ListY + 100) + 10 * Scroll THEN ListY = ListY + 1
		NEXT Scroll
		IF P$ = CHR$(27) THEN RETURN
	LOOP UNTIL P$ = CHR$(13)
	CurTile = (SelX - 1) + 12 * (SelY - 1)
RETURN

LoadBackdrop:
	Fail = FALSE
	Msgbox VIDEO, "Loading" + Backdrop$ + "..."
	Crap = DQBloadImage(2, 0, 0, Backdrop$, Pal, 0, 0)
	IF Crap THEN
		Msgbox VIDEO, "I cannot load " + Backdrop$ + "!"
		WHILE INKEY$ = "": WEND
		Fail = TRUE
		RETURN
	END IF
RETURN

LoadSet:
	Fail = FALSE
	IF FileNotFound(SetFile$) THEN
		Msgbox VIDEO, "I cannot load " + SetFile$ + "!"
		WHILE INKEY$ = "": WEND
		Fail = TRUE
		RETURN
	END IF
	Msgbox VIDEO, "Loading " + SetFile$ + "..."
	OPEN SetFile$ FOR BINARY AS #1
	IF INPUT$(4, 1) <> "STS1" THEN
		Msgbox VIDEO, SetFile$ + " is not a valid tileset file!"
		WHILE INKEY$ = "": WEND
		Fail = TRUE
		RETURN
	END IF
	Tile(0) = 24 * 8
	Tile(1) = 24
	FOR LoadTile = 0 TO 255
		TileAnim(LoadTile) = CVI(INPUT$(2, 1))
		TileFlag(LoadTile) = CVI(INPUT$(2, 1))
		AnimRate(LoadTile) = ASC(INPUT$(1, 1))
		DEF SEG = VARSEG(Tile(2))
		FOR ry = 1 TO 24
			Buffer$ = INPUT$(24, 1)
			FOR rx = 1 TO 24
				Col = ASC(MID$(Buffer$, rx, 1))
				Offs = (rx - 1) + 24 * (ry - 1)
				POKE VARPTR(Tile(2)) + Offs, Col
			NEXT rx
		NEXT ry
		DEF SEG
		DQBpoke VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * LoadTile, TileSize
	NEXT LoadTile
	CLOSE #1
	' Reset animation stats
	DEF SEG = VARSEG(AnimData(0))
	FOR Set = 0 TO 255
		POKE VARPTR(AnimData(Set)), 2
		POKE VARPTR(AnimData(Set)) + 1, 0
	NEXT Set
	DEF SEG
RETURN

Clearlevel:
	' Clear map data in base and EMS mem
	DEF SEG = VARSEG(MapData(0))
	FOR Cy = 1 TO 100: FOR cx = 1 TO 150
		Offs = (cx - 1) + 150 * (Cy - 1)
		POKE VARPTR(MapData(0)) + Offs, 0
	NEXT cx: NEXT Cy
	DEF SEG
	FOR LClear = 0 TO 1
		DQBpoke VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * LClear), 15000
	NEXT LClear
RETURN

ViewLevel:
	DQBinitText
	CLS
	Fail = FALSE
	IF VESAcheck THEN PRINT "VESA not found.": Fail = TRUE
	IF VESAsetVideoMode(1280, 1024, 8) THEN PRINT "I can't get to 1280x1024x8!": Fail = TRUE
	IF Fail THEN
		WHILE INKEY$ = "": WEND
		GOTO SkipView
	END IF
	' Render the map
	ViewX = 1
	ViewY = 1
	'$INCLUDE: 'LOADPAL.BI'
	DO
		StartX = ViewX: StartY = ViewY
		EndX = StartX + 52: EndY = StartY + 41
		IF EndX > MapXDim THEN EndX = MapXDim
		IF EndY > MapYDim THEN EndY = MapYDim
		FOR Layer = 1 TO 0 STEP -1
			DQBpeek VARSEG(MapData(0)), VARPTR(MapData(0)), 148480 + (15000 * Layer), 15000
			FOR dy = StartY TO EndY
				FOR dx = StartX TO EndX
					' Get tile number
					Offset = (dx - 1) + 150 * (dy - 1)
					DEF SEG = VARSEG(MapData(0))
					Tile = PEEK(VARPTR(MapData(0)) + Offset)
					DEF SEG
					' Draw tile
					DQBpeek VARSEG(Tile(0)), VARPTR(Tile(0)), CLNG(TileSize) * Tile, TileSize
					Tx = 24 * (((dx - 1) - (ViewX - 1)))
					Ty = 24 * (((dy - 1) - (ViewY - 1)))
					SELECT CASE Layer
						CASE 1
							FOR Py = 1 TO 24
								FOR Px = 1 TO 24
									Offset = (Px - 1) + 24 * (Py - 1)
									DEF SEG = VARSEG(Tile(2))
									Col = PEEK(VARPTR(Tile(2)) + Offset)
									DEF SEG
									VESAdrawPixelPal CLNG(Tx + (Px - 1)), CLNG(Ty + (Py - 1)), CLNG(Col)
								NEXT Px
							NEXT Py
						CASE 0
							FOR Py = 1 TO 24
								FOR Px = 1 TO 24
									Offset = (Px - 1) + 24 * (Py - 1)
									DEF SEG = VARSEG(Tile(2))
									Col = PEEK(VARPTR(Tile(2)) + Offset)
									DEF SEG
									IF Col THEN VESAdrawPixelPal CLNG(Tx + (Px - 1)), CLNG(Ty + (Py - 1)), CLNG(Col)
								NEXT Px
							NEXT Py
						END SELECT
				NEXT dx
			NEXT dy
		NEXT Layer
		SOUND 2000, .1
		P$ = "": WHILE P$ = "": P$ = INKEY$: WEND
		IF P$ = CHR$(0) + "H" AND ViewY > 1 THEN ViewY = ViewY - 1
		IF P$ = CHR$(0) + "P" AND ViewY < MapYDim - 41 THEN ViewY = ViewY + 1
		IF P$ = CHR$(0) + "K" AND ViewX > 1 THEN ViewX = ViewX - 1
		IF P$ = CHR$(0) + "M" AND ViewX < MapXDim - 52 THEN ViewX = ViewX + 1
	LOOP UNTIL P$ = CHR$(27)
	VESAsetTextMode
SkipView:
	DQBinitVGA
	'$INCLUDE: 'LOADPAL.BI'
RETURN

REM $STATIC
FUNCTION GETmemRequired& (GETfile$)

OPEN GETfile$ FOR BINARY AS #1
IF LOF(1) = 0 THEN
	CLOSE #1: KILL GETfile$
	GETmemRequired& = -1
	EXIT FUNCTION
END IF
IF INPUT$(4, 1) <> "GET1" THEN
	CLOSE #1: GETmemRequired& = -2
	EXIT FUNCTION
END IF

BoxCount = ASC(INPUT$(1, 1))
NameLength = ASC(INPUT$(1, 1))
SEEK #1, NameLength + (LOC(1) + 1)

FOR Calc = 1 TO BoxCount
	X1 = CVI(INPUT$(2, 1))
	Y1 = CVI(INPUT$(2, 1))
	X2 = CVI(INPUT$(2, 1))
	Y2 = CVI(INPUT$(2, 1))
	Xsize = X2 - X1
	Ysize = Y2 - Y1
	MemRequired& = MemRequired& + (Xsize * Ysize) + 4
NEXT Calc
CLOSE #1

GETmemRequired& = MemRequired&

END FUNCTION

FUNCTION GETprocess (GETfile$, TempLayer, TempSeg, TempOff, TempPal$)

OPEN GETfile$ FOR BINARY AS #1
IF LOF(1) = 0 THEN
	CLOSE #1: KILL GETfile$
	GETprocess = -1: EXIT FUNCTION
END IF
IF INPUT$(4, 1) <> "GET1" THEN
	CLOSE #1: GETprocess = -2
	EXIT FUNCTION
END IF

SpriteCount = ASC(INPUT$(1, 1))
IMGfile$ = INPUT$(ASC(INPUT$(1, 1)), 1)
IF DQBloadImage(TempLayer, 0, 0, IMGfile$, TempPal$, 0, 0) THEN
	CLOSE #1: GETprocess = -3
	EXIT FUNCTION
END IF

Offs& = SpriteDataOffs
FOR Calc = 1 TO NewSpriteIndex: Offs& = Offs& + SpriteSize(Calc): NEXT Calc
FOR G = 1 TO SpriteCount
	NewSpriteIndex = NewSpriteIndex + 1
	X1 = CVI(INPUT$(2, 1))
	Y1 = CVI(INPUT$(2, 1))
	X2 = CVI(INPUT$(2, 1))
	Y2 = CVI(INPUT$(2, 1))
	DQBget TempLayer, X1, Y1, X2, Y2, TempSeg, TempOff
	SpriteSize(NewSpriteIndex) = DQBsize(X1, Y1, X2, Y2)
	SpriteOffs(NewSpriteIndex) = Offs&
	DQBpoke TempSeg, TempOff, Offs&, SpriteSize(NewSpriteIndex)
	Offs& = Offs& + SpriteSize(NewSpriteIndex)
NEXT G

CLOSE #1

END FUNCTION

SUB GETsprite (Index, SpriteSeg, SpriteOff)
DQBpeek SpriteSeg, SpriteOff, SpriteOffs(Index), SpriteSize(Index)
END SUB

SUB ScrollLayer (Layer, XMove, YMove)

DIM LineBuffer(160)

DEF SEG = VARSEG(LineBuffer(0))
SELECT CASE XMove
	CASE IS < 0
		FOR Copy = 0 TO 199 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, 0, Copy)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, 0, Copy + 1)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, 0, Copy + 2)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, 0, Copy + 3)
		NEXT
		DQBscroll Layer, -1, 0
		FOR Paste = 0 TO 199 STEP 4
			DQBpset Layer, 319, Paste, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, 319, Paste + 1, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, 319, Paste + 2, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, 319, Paste + 3, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
	CASE IS > 0
		FOR Copy = 0 TO 199 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, 319, Copy)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, 319, Copy + 1)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, 319, Copy + 2)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, 319, Copy + 3)
		NEXT
		DQBscroll Layer, 1, 0
		FOR Paste = 0 TO 199 STEP 4
			DQBpset Layer, 0, Paste, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, 0, Paste + 1, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, 0, Paste + 2, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, 0, Paste + 3, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
END SELECT
SELECT CASE YMove
	CASE IS < 0
		FOR Copy = 0 TO 319 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, Copy, 0)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, Copy + 1, 0)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, Copy + 2, 0)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, Copy + 3, 0)
		NEXT
		DQBscroll Layer, 0, -1
		FOR Paste = 0 TO 319 STEP 4
			DQBpset Layer, Paste, 199, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, Paste + 1, 199, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, Paste + 2, 199, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, Paste + 3, 199, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
	CASE IS > 0
		FOR Copy = 0 TO 319 STEP 4
			POKE VARPTR(LineBuffer(0)) + Copy, DQBpoint(Layer, Copy, 199)
			POKE VARPTR(LineBuffer(0)) + Copy + 1, DQBpoint(Layer, Copy + 1, 199)
			POKE VARPTR(LineBuffer(0)) + Copy + 2, DQBpoint(Layer, Copy + 2, 199)
			POKE VARPTR(LineBuffer(0)) + Copy + 3, DQBpoint(Layer, Copy + 3, 199)
		NEXT
		DQBscroll Layer, 0, 1
		FOR Paste = 0 TO 319 STEP 4
			DQBpset Layer, Paste, 0, PEEK(VARPTR(LineBuffer(0)) + Paste)
			DQBpset Layer, Paste + 1, 0, PEEK(VARPTR(LineBuffer(0)) + (Paste + 1))
			DQBpset Layer, Paste + 2, 0, PEEK(VARPTR(LineBuffer(0)) + (Paste + 2))
			DQBpset Layer, Paste + 3, 0, PEEK(VARPTR(LineBuffer(0)) + (Paste + 3))
		NEXT Paste
END SELECT
DEF SEG

ERASE LineBuffer

END SUB

